@@ -8,10 +8,13 @@ our @ISA       = qw(Exporter);
 our @EXPORT_OK = qw/is_sanctioned set_sanction_file get_sanction_file/;
 
 use Carp;
+use Data::Validate::Sanctions::Fetcher;
 use File::stat;
+use File::ShareDir;
+use YAML::XS qw/DumpFile LoadFile/;
 use Scalar::Util qw(blessed);
 
-our $VERSION = '0.10';
+our $VERSION = '0.11';
 
 # for OO
 sub new {    ## no critic (RequireArgUnpacking)
@@ -37,6 +40,10 @@ sub get_sanction_file {
 }
 
 sub is_sanctioned {        ## no critic (RequireArgUnpacking)
+    return (get_sanctioned_info(@_))->{matched};
+}
+
+sub get_sanctioned_info {    ## no critic (RequireArgUnpacking)
     my $self = blessed($_[0]) ? shift : $instance;
 
     unless ($self) {
@@ -44,44 +51,83 @@ sub is_sanctioned {        ## no critic (RequireArgUnpacking)
         $self = $instance;
     }
 
-    my $name = join('', @_);
-    $name = uc($name);
-    $name =~ s/[[:^alpha:]]//g;
-
     my $data = $self->_load_data();
-    return 1 if grep { $_ eq $name } @$data;
 
-    # try reverse
-    if (@_ > 1) {
-        $name = join('', reverse @_);
-        $name = uc($name);
-        $name =~ s/[[:^alpha:]]//g;
-        return 1 if grep { $_ eq $name } @$data;
+    # prepare list of possible variants of names: LastnameFirstname and FirstnameLastname
+    my @name_variants = map {
+        my $name = uc(join('.*', map { my $x = $_; $x =~ s/[[:^alpha:]]//g; $x } @$_));
+        $name
+    } ([@_], @_ > 1 ? [reverse @_] : ());
+
+    for my $k (sort keys %$data) {
+        foreach my $name (@{$data->{$k}{names}}) {
+            (my $check_name = $name) =~ s/[[:^alpha:]]//g;
+            $check_name = uc($check_name);
+            for (@name_variants) {
+                return +{
+                    matched => 1,
+                    list    => $k,
+                    name    => $name,
+                } if $check_name =~ /$_/;
+            }
+        }
     }
 
-    return 0;
+    return {matched => 0};
 }
 
 sub _load_data {
     my $self          = shift;
     my $sanction_file = $self->{sanction_file};
-    my $stat          = stat($sanction_file) or croak "Can't get stat of file $sanction_file, please check it.\n";
-    return $self->{_data} if ($stat->mtime <= $self->{last_time} && $self->{_data});
+    $self->{last_time} //= 0;
+    $self->{_data} //= {};
 
-    open(my $fh, '<', $sanction_file) or croak "Can't open file $sanction_file, please check it.\n";
-    my @_data = <$fh>;
-    close($fh);
-    chomp(@_data);
-    $self->{last_time} = $stat->mtime;
-    $self->{_data}     = \@_data;
+    if (-e $sanction_file) {
+        return $self->{_data} if stat($sanction_file)->mtime <= $self->{last_time} && $self->{_data};
+        $self->{last_time} = stat($sanction_file)->mtime;
+        $self->{_data}     = LoadFile($sanction_file);
+    }
     return $self->{_data};
 }
 
+sub update_data {
+    my $self = shift;
+
+    my $new_data = Data::Validate::Sanctions::Fetcher::run();
+    $self->_load_data();
+    my $updated;
+    foreach my $k (keys %$new_data) {
+        if (ref($self->{_data}{$k}) ne 'HASH' || $self->{_data}{$k}{updated} < $new_data->{$k}{updated}) {
+            $self->{_data}{$k} = $new_data->{$k};
+            $updated = 1;
+        }
+    }
+
+    $self->_save_data if $updated;
+    return;
+}
+
+sub _save_data {
+    my $self = shift;
+
+    my $sanction_file     = $self->{sanction_file};
+    my $new_sanction_file = $sanction_file . ".tmp";
+
+    DumpFile($new_sanction_file, $self->{_data});
+
+    rename $new_sanction_file, $sanction_file or die "Can't rename $new_sanction_file to $sanction_file, please check it\n";
+    $self->{last_time} = stat($sanction_file)->mtime;
+    return;
+}
+
 sub _default_sanction_file {
-    return $ENV{SANCTION_FILE} if $ENV{SANCTION_FILE};
-    my $sanction_file = __FILE__;
-    $sanction_file =~ s/\.pm/\.csv/;
-    return $sanction_file;
+    return $ENV{SANCTION_FILE} // File::ShareDir::dist_file('Data-Validate-Sanctions', 'sanctions.yml');
+}
+
+sub last_updated {
+    my $self = shift;
+    my $list = shift;
+    return $list ? $self->{_data}->{$list}->{updated} : $self->{last_time};
 }
 
 1;
@@ -132,9 +178,30 @@ when one string is passed, please be sure last_name is before first_name.
 
 or you can pass first_name, last_name (last_name, first_name), we'll check both "$last_name $first_name" and "$first_name $last_name".
 
-return 1 for yes, 0 for no.
+retrun 1 if match is found and 0 if match is not found.
 
-it will remove all non-alpha chars and compare with the list we have.
+It will remove all non-alpha chars and compare with the list we have.
+
+=head2 get_sanctioned_info
+
+    my $result =get_sanctioned_info($last_name, $first_name);
+    print 'match: ', $result->{name}, ' on list ', $result->{list} if $result->{matched};
+
+return hashref with keys:
+    matched      1 or 0, depends if name has matched
+    list       name of list matched (present only if matched)
+    name        name of sanctioned person matched (present only if matched)
+
+It will remove all non-alpha chars and compare with the list we have.
+
+=head2 update_data
+
+Fetches latest versions of sanction lists, and updates corresponding sections of stored file, if needed
+
+=head2 last_updated
+
+Returns timestamp of stored file updated.
+If argument is provided - return timestamp when that list was updated.
 
 =head2 new
 
